from django.utils.translation import ugettext_lazy as _

from . import _callback


class Command(_callback.Command):
    help = str(_('Registers the callback with the target connectwise system.'))
    ACTION = 'create'



from django.conf.urls import url
from . import views

app_name = 'djconnectwise'

urlpatterns = [
    url(
        regex=r'^callback/$',
        view=views.CallBackView.as_view(),
        name='callback'
    ),
]



from django.utils.translation import ugettext_lazy as _
from . import _callback


class Command(_callback.Command):
    help = str(_('Deletes the callback from the target connectwise system.'))
    ACTION = 'delete'



import hashlib
import re

from django.conf import settings

_underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
_underscorer2 = re.compile('([a-z0-9])([A-Z])')
FILENAME_EXTENSION_RE = re.compile('\.([\w]*)$')


def camel_to_snake(s):
    """
    Is it ironic that this function is written in camel case, yet it
    converts to snake case? hmm..
    """
    subbed = _underscorer1.sub(r'\1_\2', s)
    return _underscorer2.sub(r'\1_\2', subbed).lower()


def snake_to_camel(snake_case_text):
    tokens = snake_case_text.split('_')
    return ''.join(word.capitalize() for word in tokens)


def get_hash(content):
    """Return the hex SHA-1 hash of the given content."""
    return hashlib.sha1(content).hexdigest()


def get_filename_extension(filename):
    """From the given filename, return the extension,
    or None if it can't be parsed.
    """
    m = FILENAME_EXTENSION_RE.search(filename)
    return m.group(1) if m else None


class DjconnectwiseSettings:
    def get_settings(self):
        # Make some defaults
        request_settings = {
            'timeout': 30.0,
            'batch_size': 50,
            'max_attempts': 3,
        }

        if hasattr(settings, 'DJCONNECTWISE_CONF_CALLABLE'):
            request_settings.update(settings.DJCONNECTWISE_CONF_CALLABLE())

        return request_settings



# Generated by Django 2.0 on 2018-08-24 14:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0072_remove_callbackentry_member'),
    ]

    operations = [
        migrations.AddField(
            model_name='boardstatus',
            name='escalation_status',
            field=models.CharField(blank=True, choices=[('NotResponded', 'Not Responded'), ('Responded', 'Responded'), ('ResolutionPlan', 'Resolution Plan'), ('Resolved', 'Resolved'), ('NoEscalation', 'No Escalation')], db_index=True, max_length=20, null=True),
        ),
        migrations.AddField(
            model_name='company',
            name='calendar',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.Calendar'),
        ),
        migrations.AddField(
            model_name='ticket',
            name='do_not_escalate_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='minutes_waiting',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='ticket',
            name='sla_expire_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='sla_stage',
            field=models.CharField(blank=True, choices=[('respond', 'Respond'), ('plan', 'Plan'), ('resolve', 'Resolve'), ('resolved', 'Resolved'), ('waiting', 'Waiting')], db_index=True, max_length=250, null=True),
        ),
    ]



from django.db.models.signals import pre_save
from django.dispatch import receiver

from .models import Ticket
import logging


logger = logging.getLogger(__name__)


@receiver(pre_save, sender=Ticket)
def handle_ticket_sla_update_pre_save(sender, instance, **kwargs):
    # Signal for updating a tickets SLA information if necessary
    try:
        old_ticket = Ticket.objects.get(id=instance.id)
        if old_ticket.status > instance.status or \
                old_ticket.status < instance.status:
            instance.calculate_sla_expiry(
                    old_status=old_ticket.status)
        elif old_ticket.priority != instance.priority:
            instance.calculate_sla_expiry()
    except Ticket.DoesNotExist:
        # This is normal when creating a new ticket.
        instance.calculate_sla_expiry()
    except TypeError:
        # This will happen during unit tests unrelated to SLAs
        # Rather than updating all the old unit tests with quite a bit of
        # SLA data they do not need at the moment, just skip SLA calculations
        # on those tests
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0040_auto_20170926_2145'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='callbackentry',
            options={'verbose_name_plural': 'Callback entries', 'verbose_name': 'Callback entry'},
        ),
        migrations.AlterModelOptions(
            name='companystatus',
            options={'verbose_name_plural': 'Company statuses'},
        ),
        migrations.AlterModelOptions(
            name='opportunitypriority',
            options={'ordering': ('name',), 'verbose_name_plural': 'opportunity priorities'},
        ),
    ]



import logging
from requests.exceptions import RequestException
import django
from djconnectwise.api import SystemAPIClient
from djconnectwise.models import CallBackEntry
from djconnectwise.utils import DjconnectwiseSettings


logger = logging.getLogger(__name__)


class CallBackHandler:
    CALLBACK_ID = None
    CALLBACK_TYPE = None

    def __init__(self, *args, **kwargs):
        self.system_client = SystemAPIClient()

        self.settings = DjconnectwiseSettings().get_settings()

        if not self.CALLBACK_ID:
            raise NotImplementedError('CALLBACK_ID must be assigned a value')

        if not self.CALLBACK_TYPE:
            raise NotImplementedError('CALLBACK_TYPE must be assigned a value')

    def _create_callback(self, callback_id, callback_type):
        """
        Registers a callback with the target connectwise system.
        Creates and returns a local CallBackEntry instance.
        """
        params = {
            'url': '{}{}'.format(
                self.settings['callback_host'],
                self.settings['callback_url']
            ),
            'objectId': callback_id,
            'type': callback_type,
            'level': 'owner',
            'description': 'Kanban application {} callback'.format(
                callback_type
            )
        }

        entry_json = self.system_client.create_callback(params)

        if entry_json:
            entry = None
            try:
                entry = CallBackEntry.objects.create(
                    id=entry_json['id'],
                    url=entry_json['url'],
                    object_id=entry_json['objectId'],
                    level=entry_json['level'],
                    description=entry_json['description'],
                    callback_type=entry_json['type'],
                    inactive_flag=False
                )

            except django.db.utils.Error as e:
                logger.error('CallBackEntry Creation Failed: {}'.format(e))
                self.system_client.delete_callback(entry_json['id'])

            return entry

    def create(self):
        """
        Registers the callback with the target connectwise system.
        Creates and returns a local CallBackEntry instance.
        """
        return self._create_callback(self.CALLBACK_ID,
                                     self.CALLBACK_TYPE)

    def get_callbacks(self):
        """
        Returns a list of dict callback entries that
        are registered with the target system.
        """
        results = []
        page = 1
        while True:
            logger.info(
                'Fetching callback records, batch {}'.format(page)
            )
            page_records = self.system_client.get_callbacks(
                page=page, page_size=self.settings['batch_size'],
            )
            results += page_records
            page += 1
            if len(page_records) < self.settings['batch_size']:
                # This page wasn't full, so there's no more records after
                # this page.
                break
        return results

    def delete(self):
        """
        Removes the callback from connectwise and removes the local record.
        """
        entry_qset = CallBackEntry.objects.filter(
            callback_type=self.CALLBACK_TYPE)

        entries = {e.id: e for e in entry_qset}
        host = self.settings['callback_host']
        # Only delete a callback that starts with our expected hostname, so
        # we don't whack another integration's callbacks.
        api_entries = [
            e for e in self.get_callbacks() if
            e['type'] == self.CALLBACK_TYPE and e['url'].startswith(host)
        ]

        for entry in api_entries:
            entry_id = entry['id']
            try:
                # Only delete the DB entry once CW has
                # accepted our delete request.
                self.system_client.delete_callback(entry_id)
                if entry_id in entries:
                    entries[entry_id].delete()

                logger.info('Deleted callback {}.'.format(entry_id))
            except RequestException as e:
                msg = 'Failed to remove callback: {}: {}'
                logger.warning(msg.format(entry_id, e))


class TicketCallBackHandler(CallBackHandler):
    CALLBACK_ID = 1
    CALLBACK_TYPE = CallBackEntry.CALLBACK_TYPES.ticket


class ProjectCallBackHandler(CallBackHandler):
    CALLBACK_ID = 2
    CALLBACK_TYPE = CallBackEntry.CALLBACK_TYPES.project


class CompanyCallBackHandler(CallBackHandler):
    CALLBACK_ID = 3
    CALLBACK_TYPE = CallBackEntry.CALLBACK_TYPES.company


class OpportunityCallBackHandler(CallBackHandler):
    CALLBACK_ID = 4
    CALLBACK_TYPE = CallBackEntry.CALLBACK_TYPES.opportunity



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0044_auto_20171222_1700'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='actual_hours',
            field=models.DecimalField(max_digits=6, blank=True, null=True, decimal_places=2),
        ),
        migrations.AddField(
            model_name='project',
            name='budget_hours',
            field=models.DecimalField(max_digits=6, blank=True, null=True, decimal_places=2),
        ),
        migrations.AddField(
            model_name='project',
            name='scheduled_hours',
            field=models.DecimalField(max_digits=6, blank=True, null=True, decimal_places=2),
        ),
    ]
